#!/usr/bin/perl
#
# linkat-hotspot scripts
# Aleix Solé Romeu <asole@itechgrup.com>
# Common stuff and configuration file reference.
#

BEGIN { push(@INC, ".."); };
use DBI;
use DBD::mysql;
use WebminCore;
use Data::Dumper;

init_config();

#
# Reads DB connection config from the proper file
#
sub read_hotspot_config
{
	my %db_config;
	my $config_file = $config{'hotspot_conf'};
	read_file($config_file, \%db_config);

	return %db_config;
}

#
# Set up DB connection
#
sub db_connect
{
	my ($db_config) = @_;
	my $dbh;

	my $mysql_database = $db_config->{'mysql_database'};
	my $mysql_user = $db_config->{'mysql_user'};
	my $mysql_password = $db_config->{'mysql_password'};
	
	# Connect to the database.
	$dbh = DBI->connect("DBI:mysql:database=$mysql_database;host=localhost",
                       "$mysql_user", "$mysql_password",
                       {'RaiseError' => 1});
	return $dbh;
}


sub get_groups_hash
{
	my ($dbh) = @_;
	my %groups;

	my $query = "select id,name,settings from groups";
	$sth = $dbh->prepare($query);
	$sth->execute();
	my ($groupname, $value);
	while (my $ref = $sth->fetchrow_hashref()) {
		$id = $ref->{'id'};
		$groupname = $ref->{'name'};
		$value = $ref->{'settings'};
			
		$groups{$groupname}{'id'} = $id;	
		my @tokens = split (/\;/, $value);
		foreach (@tokens)
		{
			my ($key,$value) = split (/=/, $_, 2);
			$groups{$groupname}{'settings'}{$key} = $value;
		}
	}

	return %groups;
}

#
# Return users within a group 
#
sub get_users_from_group
{
	my ($dbh, $gid) = @_;

	my %users;

	my $query = "select users.id as userid,groups.id,groups.name as groupname,users.name as username from groups left join usergroup on groups.id=group_id left join users on user_id=users.id where groups.id='$gid'";
	$sth = $dbh->prepare($query);
	$sth->execute();
	my ($gname, $value);
	while (my $ref = $sth->fetchrow_hashref()) {
		$name = $ref->{'username'};
		$uid = $ref->{'userid'};	
		$users{$name} = $uid;
	}

	return %users;
}

#
# Return all users in a hash
#
sub get_allusers
{
	my ($dbh, $gid) = @_;

	my %users;

	my $query = "select * from users";
	$sth = $dbh->prepare($query);
	$sth->execute();
	my ($name,$id);
	while (my $ref = $sth->fetchrow_hashref()) {
		$id = $ref->{'id'};
		$name = $ref->{'name'};
			
		$users{$name}{'name'} = $name;
		$users{$name}{'id'} = $id;
	}

	return %users;
}

#
# Global settings parsing
#
sub get_globalsettings
{
	my ($dbh) = @_;
	my %globalsettings;

	my $query = "select * from globalsettings";
	my $sth = $dbh->prepare($query);
	$sth->execute();
	my ($key, $value);
	while (my $ref = $sth->fetchrow_hashref()) {
		$key = $ref->{'key'};
		$value = $ref->{'value'};
	
		$globalsettings{$key} = $value;
	}

	return %globalsettings;
}


sub add_user_to_group
{
	my ($dbh, $uid, $gid) = @_;

       	$sth = $dbh->prepare("SELECT count(*) FROM usergroup where user_id='$uid' and group_id='$gid'");
       	$sth->execute();
       	if ($sth->fetch()->[0]) {
		# print " User $uid already registered in group $gid<br>";
       	}
       	else
       	{
		$dbh->do("INSERT INTO usergroup (user_id, group_id) VALUES (?,?)", undef, $uid, $gid);
       	}
}



sub remove_user_from_group
{
	my ($dbh, $uid, $gid) = @_;

       	$sth = $dbh->prepare("SELECT count(*) FROM usergroup where user_id='$uid' and group_id='$gid'");
       	$sth->execute();
       	if ($sth->fetch()->[0]) {
		$dbh->do("DELETE FROM usergroup where user_id='$uid' and group_id='$gid'");
       	}
}

sub update_group_acl
{
	my ($dbh, $gid, $acl) = @_;

	my $acl_txt = "";
	foreach (keys %{$acl})
	{
		$acl_txt .= $_ . "=" . $acl->{$_} .";";
	}	
	chop $acl_txt;

	print "Updating ACL for group #$gid<br>";
	$dbh->do("UPDATE groups set groups.settings='$acl_txt' where groups.id='$gid'");
}

sub update_globalsettings
{
	my ($dbh, $settings) = @_;

	foreach (keys %{$settings})
	{
		print "Updating global setting '$_'... ";
		$sth = $dbh->prepare("SELECT * FROM globalsettings where globalsettings.key='$_'");
		$sth->execute();
		if (my $ref = $sth->fetchrow_hashref()) {
			print "value found, updating...";
			$value = $settings->{$_};
			$dbh->do("UPDATE globalsettings set globalsettings.value='$value' where globalsettings.key='$_'");
			print "OK. <br>";
		}
		else
		{
			print "value not found in DB, creating...";
			$dbh->do("INSERT INTO globalsettings (globalsettings.key,globalsettings.value) VALUES (?,?)", undef, $_, $value);
		}
	}	
}

sub create_group
{
	my ($dbh, $groupname, $newgroup_perms) = @_;

	print "Setting up group $groupname\n";
	my $group_id;
	# Check if group already exists in db
	$sth = $dbh->prepare("SELECT * FROM groups where name='$groupname'");
	$sth->execute();
	if (my $ref = $sth->fetchrow_hashref()) {
		print "Error! Group already exists\n";
		$group_id = $ref->{'id'};
	}
	else
	{
		$dbh->do("INSERT INTO groups (name,settings) VALUES (?,?)", undef, $groupname, $newgroup_perms);
		$group_id = $dbh->{'mysql_insertid'};
		print "Group '$groupname' created with id $group_id\n"
	}
}


sub delete_group
{
	my ($dbh, $gid) = @_;

	print "Deleting group $gid\n";
	my $group_id;
	# Check if group already exists in db
	$sth = $dbh->prepare("SELECT * FROM groups where id='$gid'");
	$sth->execute();
	if (my $ref = $sth->fetchrow_hashref()) {

                $dbh->do("DELETE FROM usergroup where group_id='$gid'");
		$dbh->do("DELETE FROM groups where id='$gid'");
		print "Group deleted.\n"
	}
	else
	{
		print "Error! Group id doesn't exist in database?\n";
	}
}

1;
